package test;

import garage.ExecutionFailedException;
import garage.Task;
import garage.TestBench;
import org.junit.Before;
import org.junit.FixMethodOrder;
import org.junit.Test;
import vehicle.Vehicle;
import vehicle.VehicleModel;
import vehicle.part.gearbox.Gear;
import vehicle.part.pedal.PedalAccelerator;
import vehicle.part.pedal.PedalPosition;

import java.util.Arrays;

import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.assertNull;

public class TestBenchTest {

    private TestBench testBench;
    private Vehicle fiat500;
    private Vehicle peugeot5008;
    private Task task;

    @Before
    public void setUp(){
        testBench = new TestBench();

        fiat500 = new Vehicle(VehicleModel.FIAT_500);
        peugeot5008 = new Vehicle(VehicleModel.PEUGEOT_5008);
        task = new Task(fiat500.getPedals().get(0), "setState", Arrays.asList(50d), 5);
    }

    @Test
    public void addVehicleTest(){
        Assert.assertNull(testBench.getVehicle());
        testBench.addVehicle(fiat500);
        TestCase.assertEquals(fiat500, testBench.getVehicle());
    }

    @Test
    public void addVehicleImpossibleTest(){
        Assert.assertNull(testBench.getVehicle());
        testBench.addVehicle(fiat500);
        testBench.addVehicle(peugeot5008);
        TestCase.assertEquals(fiat500, testBench.getVehicle());
    }

    @Test
    public void removeVehicleTest(){
        testBench.addVehicle(fiat500);
        TestCase.assertEquals(fiat500, testBench.getVehicle());
        testBench.removeVehicle();
        Assert.assertNull(testBench.getVehicle());
    }

    @Test
    public void removeVehicleImpossibleTest(){
        testBench.removeVehicle();
        Assert.assertNull(testBench.getVehicle());
    }

    @Test
    public void swapVehicleTest(){
        testBench.addVehicle(fiat500);
        TestCase.assertEquals(fiat500, testBench.getVehicle());
        Vehicle swapped = testBench.swapVehicle(peugeot5008);
        TestCase.assertEquals(fiat500, swapped);
        TestCase.assertEquals(peugeot5008, testBench.getVehicle());
    }

    @Test
    public void swapVehicleImpossibleTest(){
        Vehicle swapped = testBench.swapVehicle(peugeot5008);
        Assert.assertNull(swapped);
        Assert.assertNull(testBench.getVehicle());
    }

    @Test
    public void addTaskTest(){
        TestCase.assertEquals(0, testBench.getTasks().size());
        testBench.addTask(task);
        TestCase.assertEquals(1, testBench.getTasks().size());
        TestCase.assertTrue(testBench.getTasks().contains(task));
    }

    @Test
    public void removeTaskTest(){
        testBench.addTask(task);
        TestCase.assertEquals(1, testBench.getTasks().size());
        testBench.removeTask(task);
        TestCase.assertEquals(0, testBench.getTasks().size());
    }

    @Test
    public void runTest(){
        testBench.addVehicle(fiat500);
        testBench.addTask(task);
        TestCase.assertEquals(0d, fiat500.getPedals().get(0).getState());
        testBench.run(10);
        TestCase.assertEquals(50d, fiat500.getPedals().get(0).getState());
    }

    @Test
    public void runWrongTaskTest(){
        testBench.addVehicle(fiat500);
        task = new Task(fiat500.getPedals().get(0), "setStat", Arrays.asList(50d), 5);
        testBench.addTask(task);
        testBench.run(10);
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    @Test
    public void scenario1Test(){
        testBench.addVehicle(fiat500);
        testBench.addTask(new Task(fiat500.getPedals().get(PedalPosition.CLUTCH.getValue()), "setState", Arrays.asList(70d), 2));
        testBench.addTask(new Task(fiat500.getGearBox(), "gearUp", null, 3));
        testBench.addTask(new Task(fiat500.getPedals().get(PedalPosition.CLUTCH.getValue()), "setState", Arrays.asList(0d), 4));

        TestCase.assertEquals(Gear.NEUTRAL, fiat500.getGearBox().getCurrentGear());
        testBench.run(10);
        TestCase.assertEquals(Gear.FIRST, fiat500.getGearBox().getCurrentGear());
    }

}
